'use strict';

const HtmlWebpackPlugin = require('html-webpack-plugin');
const fs = require('fs');
const buildChunksListingFileName = './built-chunks.txt';

class WebpackNoteBuildChunksPlugin {
  apply(compiler) {
    if (compiler.hooks) {
      compiler.hooks.compilation.tap('NoteBuildChunksPlugin', compilation => {
        if (compilation.hooks.htmlWebpackPluginAlterAssetTags) {
          compilation.hooks.htmlWebpackPluginAlterAssetTags.tapAsync(
            'NoteBuildChunksPlugin',
            (data, cb) => {
              data.head = this.noteChunkFiles(data.head);
              data.body = this.noteChunkFiles(data.body);
              return cb(null, data);
            }
          );
        } else if (HtmlWebpackPlugin && HtmlWebpackPlugin.getHooks) {
          const hooks = HtmlWebpackPlugin.getHooks(compilation);
          hooks.alterAssetTags.tapAsync('NoteBuildChunksPlugin', (data, cb) => {
            data.assetTags.scripts = this.noteChunkFiles(
              data.assetTags.scripts
            );
            data.assetTags.styles = this.noteChunkFiles(data.assetTags.styles);
            data.assetTags.meta = this.noteChunkFiles(data.assetTags.meta);
            return cb(null, data);
          });
        } else {
          throw new Error('Cannot find appropriate compilation hook');
        }
      });
    } else {
      compiler.plugin('compilation', function(compilation) {
        compilation.plugin(
          'html-webpack-plugin-alter-asset-tags',
          (htmlPluginData, callback) => {
            htmlPluginData.head = this.noteChunkFiles(htmlPluginData.head);
            htmlPluginData.body = this.noteChunkFiles(htmlPluginData.body);
            return callback(null, htmlPluginData);
          }
        );
      });
    }
  }
  noteChunkFiles(assets) {
    return assets.map(asset => {
      if (
        asset.tagName &&
        asset.tagName === 'script' &&
        asset.attributes &&
        asset.attributes.src
      ) {
        // Writing source of all the chunks
        // to built-chunks.txt which is
        // later references to be written
        // to the index.html file
        fs.appendFileSync(
          buildChunksListingFileName,
          '\n' + asset.attributes.src
        );
      }
      return asset;
    });
  }
}

exports.WebpackNoteBuildChunksPlugin = WebpackNoteBuildChunksPlugin;
