'use strict';

const fs = require('fs-extra');

const indexFilePath = process.cwd() + '/build/index.html';
const buildChunksListingFileName = './built-chunks.txt';

/**
 * This function is responsible for
 * for writing individual chunk
 * files as script tags into
 * the index.html file.
 */
module.exports = function writeChunksToHtml() {
  console.log('Writing the chunk files to the index.html file');
  let scriptTags = '';
  let fileNames = getIndividualFileNamesFromChunkListFile();
  fileNames.map(fileName => {
    // Adding script tag if file isn't already imported.
    if (!fileHasText(indexFilePath, fileName)) {
      scriptTags += `<script src="${fileName}" ${
        fileName.includes('.es5.') ? 'nomodule' : ''
      }></script>`;
    } else {
      // Adding nonodule attribute if the file
      // is already imported, has `.es5.` in the
      // file name and does not have nomodule
      // attribute.
      if (fileName.includes('.es5.')) {
        addNoModuleAttributeToExistingScriptTag(fileName);
      }
    }
  });
  if (scriptTags.length) {
    fs.readFile(indexFilePath, function(err, data) {
      if (!err) {
        // Writing the Script tags before the
        // closing body tag (</body>)
        const closeBodyTagPosition = data.lastIndexOf('</body>');
        writeToHtml(closeBodyTagPosition, scriptTags);
      }
    });
  }
};

/**
 * This function is responsible for writing
 * the provided tags into the index.html file
 * at the given position / index.
 * @param {Number} position The position where we want to inject the tags
 *                          (right now it's closing body tag Refer
 *                          Line No.: 24 in this file)
 * @param {String} tags The minified script tags to be written in the HTML file.
 */
function writeToHtml(position, tags) {
  fs.readFile(indexFilePath, function read(err, data) {
    if (err) throw err;
    let file_content = data.toString();
    file_content = file_content.substring(position);
    const file = fs.openSync(indexFilePath, 'r+');
    const bufferedText = new Buffer(tags + file_content, 'utf-8');
    fs.writeSync(file, bufferedText, 0, bufferedText.length, position);
    fs.close(file);
  });
}

/**
 * This function is responsible for
 * reading the built-chunks.txt file
 * and fetching the file names from
 * the file.
 * @returns Array
 */
function getIndividualFileNamesFromChunkListFile() {
  let data = fs.readFileSync(buildChunksListingFileName, 'utf8');
  data = data.split('\n');
  return data.filter(fileName => fileName.length > 0);
}

/**
 * This function is responsible for
 * searching given string in the
 * given file.
 * @param {String} fileName File Where you want to search the text
 * @param {String} searchString Text to Search
 * @returns
 */
function fileHasText(fileName, searchString) {
  let data = fs.readFileSync(fileName, 'utf8');
  return data.lastIndexOf(searchString) > -1;
}

/**
 * This functions adds nomodule attribute
 * to all the scripts tags in the index.html
 * file having src as the provided filename
 * for the script tags.
 * @param {String} fileName FileName of the Script Src Attribute
 */
function addNoModuleAttributeToExistingScriptTag(fileName) {
  let data = fs.readFileSync(indexFilePath, 'utf8');
  data = data.replace(`"${fileName}"`, `"${fileName}" nomodule`);
  fs.writeFileSync(indexFilePath, data);
}
